<?php

namespace Drupal\fieldable_panels_panes\Plugin\migrate\source\d7;

use Drupal\migrate\Row;
use Drupal\migrate_drupal\Plugin\migrate\source\d7\FieldableEntity;

// cspell:ignore fpid

/**
 * Drupal 7 FPP source from database.
 *
 * Available configuration keys:
 * - fpp_type: The FPP types to get from the source - can be a string or an
 *   array. If not declared then FPPs of all types will be retrieved.
 *
 * Examples:
 *
 * @code
 * source:
 *   plugin: d7_fieldable_panels_pane
 *   fpp_type: thing
 * @endcode
 *
 * In this example FPPs of type "thing" are retrieved from the source database.
 *
 * @code
 * source:
 *   plugin: d7_fieldable_panels_pane
 *   fpp_type: [thing, test]
 * @endcode
 *
 * In this example FPPs of type page and test are retrieved from the source
 * database.
 *
 * For additional configuration keys, refer to the parent classes.
 *
 * @see \Drupal\migrate\Plugin\migrate\source\SqlBase
 * @see \Drupal\migrate\Plugin\migrate\source\SourcePluginBase
 *
 * @MigrateSource(
 *   id = "d7_fieldable_panels_pane",
 *   source_module = "fieldable_panels_panes"
 * )
 */
class FieldablePanelsPane extends FieldableEntity {

  /**
   * The join options between the FPP and the FPP revisions table.
   */
  const JOIN = 'fpp.vid = fppr.vid';

  /**
   * {@inheritdoc}
   */
  public function query() {
    // Select FPP in its last revision.
    $query = $this->select('fieldable_panels_panes_revision', 'fppr')
      ->fields('fpp', [
        'fpid',
        'vid',
        'bundle',
        'title',
        'link',
        'path',
        'reusable',
        'admin_title',
        'admin_description',
        'category',
        'view_access',
        'edit_access',
        'language',
        'created',
        'changed',
        'uuid',
      ])
      ->fields('fppr', [
        'fpid',
        'vid',
        'timestamp',
        'uid',
        'title',
        'log',
        'vuuid',
      ]);
    $query->innerJoin('fieldable_panels_panes', 'fpp', static::JOIN);

    if (isset($this->configuration['fpp_type'])) {
      $query->condition('fpp.bundle', (array) $this->configuration['fpp_type'], 'IN');
    }

    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public function prepareRow(Row $row) {
    $fpid = $row->getSourceProperty('fpid');
    $vid = $row->getSourceProperty('vid');
    $type = $row->getSourceProperty('bundle');

    // If this entity was translated using Entity Translation, we need to get
    // its source language to get the field values in the right language.
    $entity_translatable = $this->isEntityTranslatable('fieldable_panels_pane') && (int) $this->variableGet('language_content_type_' . $type, 0) === 4;
    $source_language = $this->getEntityTranslationSourceLanguage('fieldabpe_panels_pane', $fpid);
    $language = $entity_translatable && $source_language ? $source_language : $row->getSourceProperty('language');

    // Get Field API field values.
    foreach ($this->getFields('fieldable_panels_pane', $type) as $field_name => $field) {
      // Ensure we're using the right language if the entity and the field are
      // translatable.
      $field_language = $entity_translatable && $field['translatable'] ? $language : NULL;
      $row->setSourceProperty($field_name, $this->getFieldValues('fieldable_panels_pane', $field_name, $fpid, $vid, $field_language));
    }

    // If the title was replaced by a real field using the Drupal 7 Title
    // module, use the field value instead of the node title.
    if ($this->moduleExists('title')) {
      $title_field = $row->getSourceProperty('title_field');
      if (isset($title_field[0]['value'])) {
        $row->setSourceProperty('title', $title_field[0]['value']);
      }
    }

    return parent::prepareRow($row);
  }

  /**
   * {@inheritdoc}
   */
  public function fields() {
    $fields = [
      // From the main FPP table.
      'fpid' => $this->t('FPP ID'),
      'vid' => $this->t('Revision ID'),
      'bundle' => $this->t('Bundle'),
      'title' => $this->t('Title'),
      'link' => $this->t('Link'),
      'path' => $this->t('Path'),
      'reusable' => $this->t('Reusable'),
      'admin_title' => $this->t('Admin title'),
      'admin_description' => $this->t('Admin description'),
      'category' => $this->t('Category'),
      'view_access' => $this->t('View access'),
      'edit_access' => $this->t('Edit access'),
      'language' => $this->t('Language (fr, en, ...)'),
      'created' => $this->t('Created timestamp'),
      'changed' => $this->t('Modified timestamp'),
      'uuid' => $this->t('UUID'),
      // From the FPP revisions table.
      'timestamp' => $this->t('The timestamp the latest revision of this FPP was created.'),
      'uid' => $this->t('FPP authored by (uid)'),
      'log' => $this->t('The revision log.'),
    ];
    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function getIds() {
    $ids['fpid']['type'] = 'integer';
    $ids['fpid']['alias'] = 'fpp';
    return $ids;
  }

}
